import json
from django.test import TestCase
import datetime
from django.utils import timezone
from polls.models.question import Question
from django.urls import reverse


# class QuestionModelTest(TestCase):

#     def test_was_published_recently_with_future_question(self):
#         """
#           was_published_recently() returns False for questions whose pub_date
#           is in the future.
#         """
#         future_date = timezone.now() + datetime.timedelta(days=30)
#         future_question = Question(
#             question_text="testing question", pub_date=future_date)
#         self.assertIs(future_question.was_published_recently(), False)

#     def test_was_published_recently_with_old_question(self):
#         """
#           was_published_recently() returns False for questions whose pub_date
#           is older than 1 day.
#         """
#         old_date = timezone.now() - datetime.timedelta(days=5)
#         old_question = Question(
#             question_text="testing question", pub_date=old_date)
#         self.assertIs(old_question.was_published_recently(), False)

#     def test_was_published_recently_with_recent_question(self):
#         """
#         was_published_recently() returns True for questions whose pub_date
#         is within the last day.
#         """
#         current_date = timezone.now() - datetime.timedelta(hours=23, minutes=59, seconds=59)
#         current_question = Question(
#             question_text="testing question", pub_date=current_date)
#         self.assertIs(current_question.was_published_recently(), True)


def create_question(question_text, days):
    """
    Create a question with the given `question_text` and published the
    given number of `days` offset to now (negative for questions published
    in the past, positive for questions that have yet to be published).
    """
    time = timezone.now() + datetime.timedelta(days=days)
    return Question.objects.create(question_text=question_text, pub_date=time)


class QuestionIndexViewTests(TestCase):
    def test_no_questions(self):
        """
        If no questions exist, an appropriate message is displayed.
        """
        response = self.client.get(reverse("polls:index"))
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, "No polls are available.")
        self.assertQuerySetEqual(response.context["questions"], [])

    def test_past_question(self):
        """
        Questions with a pub_date in the past are displayed on the
        index page.
        """
        question = create_question(question_text="Past question.", days=-1)
        response = self.client.get(reverse("polls:index"))
        self.assertQuerySetEqual(
            response.context["questions"],
            [question],
        )

    def test_future_question(self):
        """
        Questions with a pub_date in the future aren't displayed on
        the index page.
        """
        create_question(question_text="Future question.", days=30)
        response = self.client.get(reverse("polls:index"))
        self.assertContains(response, "No polls are available.")
        self.assertQuerySetEqual(response.context["questions"], [])

    def test_future_question_and_past_question(self):
        """
        Even if both past and future questions exist, only past questions
        are displayed.
        """
        question = create_question(question_text="Past question.", days=-30)
        create_question(question_text="Future question.", days=30)
        response = self.client.get(reverse("polls:index"))
        self.assertQuerySetEqual(
            response.context["questions"],
            [question],
        )

    def test_two_past_questions(self):
        """
        The questions index page may display multiple questions.
        """
        question1 = create_question(question_text="Past question 1.", days=-30)
        question2 = create_question(question_text="Past question 2.", days=-5)
        response = self.client.get(reverse("polls:index"))
        self.assertQuerySetEqual(
            response.context["questions"],
            [question2, question1],
        )
